==> rtp-1.2.0.tar.gz: rtp_1_2_0/util/TimerContainer.cc <==
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const TimerContainer_cc_Version =
    "$Id: TimerContainer.cc,v 1.12 2000/12/18 23:49:06 bko Exp $";


#include "VMissingDataException.hxx"


using namespace Vocal;
using TimeAndDate::TimerContainer;
using TimeAndDate::TimerEntry;
using TimeAndDate::milliseconds_t;
using TimeAndDate::TimerEntryId;


template < class Msg >
TimerContainer < Msg > ::TimerContainer()
{
}



template < class Msg >
TimerContainer < Msg > ::~TimerContainer()
{
}



template < class Msg >
TimerEntryId
TimerContainer < Msg > ::add(
    Sptr < Msg > msg,
    milliseconds_t relativeTime
)
{
    TimerEntry < Msg > newTimerEntry(msg, relativeTime);

    list < TimerEntry < Msg > > ::reverse_iterator rit;
    for ( rit = timerList_.rbegin(); rit != timerList_.rend(); rit++ )
    {
        if ( newTimerEntry >= *rit )
        {
            timerList_.insert(rit.base(), newTimerEntry);
            break;
        }
    }
    if ( rit == timerList_.rend() )
    {
        timerList_.push_front(newTimerEntry);
    }

    return ( newTimerEntry.getId() );
}


template < class Msg >
bool
TimerContainer < Msg > ::cancel(TimerEntryId msg_id)
{
    list < TimerEntry < Msg > > ::iterator it;

    for ( it = timerList_.begin(); it != timerList_.end(); it++	)
    {
        if ( (*it).getId() == msg_id )
        {
            timerList_.erase(it);
            return ( true );
        }
    }
    return ( false );
}


template < class Msg >
int
TimerContainer < Msg > ::getTimeout()
{
    int timeout = timerList_.size() > 0
                  ? timerList_.front().getTimeout()
                  : -1;

    // Nothing in the list, so return infinite timeout (-1).
    //
    return ( timeout );
}


template < class Msg >
TimerEntryId
TimerContainer < Msg > ::getFirstTimerEntryId()
{
    TimerEntryId id = timerList_.size() > 0
                      ? timerList_.front().getId()
                      : 0;
    return ( id );
}


template < class Msg >
bool
TimerContainer < Msg > ::messageAvailable()
{
    bool msgAvailable = ( timerList_.size() > 0 && timerList_.front().hasExpired() );

    return ( msgAvailable );
}


template < class Msg >
Sptr < Msg >
TimerContainer < Msg > ::getMessage()
throw ( VMissingDataException )
{
    if ( !messageAvailable() )
    {
        throw VMissingDataException("TimerContainer::getMessage: no message available.",
                                    __FILE__, __LINE__);
    }

    Sptr < Msg > msg = timerList_.front().getMessage();
    timerList_.pop_front();

    return ( msg );
}


template < class Msg >
ostream &
TimerContainer < Msg > ::writeTo(ostream & out) const
{
    out << "timer container (size: " << timerList_.size() << ") = { ";
    bool first = true;

    list < TimerEntry < Msg > > ::const_iterator it;

    for ( it = timerList_.begin(); it != timerList_.end(); it++ )
    {
        if ( first )
        {
            out << " (";
            first = false;
        }
        else
        {
            out << ", (";
        }

        out << *it << " )";
    }

    return ( out << " }" );
}


template < class Msg >
unsigned int
TimerContainer < Msg > ::size() const
{
    return ( timerList_.size() );
}

==> rtp-1.3.0.tar.gz: rtp-1.3.0/util/TimerContainer.cc <==
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const TimerContainer_cc_Version =
    "$Id: TimerContainer.cc,v 1.15 2001/08/03 21:46:57 icahoon Exp $";


template < class Msg >
TimerContainer < Msg > ::TimerContainer()
{
}



template < class Msg >
TimerContainer < Msg > ::~TimerContainer()
{
}



template < class Msg >
TimerEntryId
TimerContainer < Msg > ::add(
    Sptr < Msg > msg,
    milliseconds_t when,
    bool relative
)
{
    TimerEntry < Msg > newTimerEntry(msg, when, relative);

    postAdd(newTimerEntry);
    
    return ( newTimerEntry.getId() );
}


template < class Msg >
TimerEntryId
TimerContainer < Msg > ::add(
    Sptr < Msg > msg,
    const TimeVal & when,
    bool relative
)
{
    TimerEntry < Msg > newTimerEntry(msg, when, relative);

    postAdd(newTimerEntry);

    return ( newTimerEntry.getId() );
}


template < class Msg >
bool
TimerContainer < Msg > ::cancel(TimerEntryId msg_id)
{
    typename list < TimerEntry < Msg > > ::iterator it;

    for ( it = timerList_.begin(); it != timerList_.end(); it++	)
    {
        if ( (*it).getId() == msg_id )
        {
            timerList_.erase(it);
            return ( true );
        }
    }
    return ( false );
}


template < class Msg >
int
TimerContainer < Msg > ::getTimeout()
{
    int timeout = timerList_.size() > 0
                  ? timerList_.front().getTimeout()
                  : -1;

    // Nothing in the list, so return infinite timeout (-1).
    //
    return ( timeout );
}


template < class Msg >
TimerEntryId
TimerContainer < Msg > ::getFirstTimerEntryId()
{
    TimerEntryId id = timerList_.size() > 0
                      ? timerList_.front().getId()
                      : 0;
    return ( id );
}


template < class Msg >
bool
TimerContainer < Msg > ::messageAvailable()
{
    bool msgAvailable = ( timerList_.size() > 0 && timerList_.front().hasExpired() );

    return ( msgAvailable );
}


template < class Msg >
Sptr < Msg >
TimerContainer < Msg > ::getMessage()
throw ( VMissingDataException )
{
    if ( !messageAvailable() )
    {
        throw VMissingDataException("TimerContainer::getMessage: no message available.",
                                    __FILE__, __LINE__);
    }

    Sptr < Msg > msg = timerList_.front().getMessage();
    timerList_.pop_front();

    return ( msg );
}


template < class Msg >
ostream &
TimerContainer < Msg > ::writeTo(ostream & out) const
{
    out << "timer container (size: " << timerList_.size() << ") = { ";
    bool first = true;

    typename list < TimerEntry < Msg > > ::const_iterator it;

    for ( it = timerList_.begin(); it != timerList_.end(); it++ )
    {
        if ( first )
        {
            out << " (";
            first = false;
        }
        else
        {
            out << ", (";
        }

        out << *it << " )";
    }

    return ( out << " }" );
}


template < class Msg >
unsigned int
TimerContainer < Msg > ::size() const
{
    return ( timerList_.size() );
}


template < class Msg >
void    
TimerContainer < Msg > ::postAdd(const TimerEntry<Msg> & newTimerEntry)
{
    typename list < TimerEntry < Msg > > ::reverse_iterator rit;
    for ( rit = timerList_.rbegin(); rit != timerList_.rend(); rit++ )
    {
        if ( newTimerEntry >= *rit )
        {
            timerList_.insert(rit.base(), newTimerEntry);
            break;
        }
    }
    if ( rit == timerList_.rend() )
    {
        timerList_.push_front(newTimerEntry);
    }
}
